import hashlib

import requests

from .exceptions import InvalidPrefix


def _get_sha1(str_input):
    encoded_input = str_input.encode()
    return hashlib.sha1(encoded_input)


def _get_hexdigest(hashed):
    return hashed.hexdigest()


def _get_pwned(prefix):
    base_url = "https://api.pwnedpasswords.com/range/{}"
    response = requests.get(base_url.format(prefix))
    if response.status_code == 400:
        raise InvalidPrefix("An invalid prefix was supplied.", prefix=prefix)
    return response.text.split()


def _check_pwned(suffix, pwned_suffixs):
    for pwned in pwned_suffixs:
        suffix_, hits = pwned.split(":")
        if suffix.upper() == suffix_.upper():
            return int(hits)
    return 0



class BaseException(IOError):
    """An ambiguous exception was raised."""

    def __init__(self, *args, **kwargs):
        """Initialize BaseException."""
        self.prefix = kwargs.pop("prefix", None)
        super(BaseException, self).__init__(*args, **kwargs)


class InvalidPrefix(BaseException):
    """An invalid prefix was supplied."""



"""
powned
------

Powned is a python package allowing you to check if your password has
been recovered in a data breach.

Basic usage:

    >>> import powned
    >>> powned.check('password1')
    2413945
"""
from .api import check  # noqa F401



from .password import _check_pwned, _get_hexdigest, _get_pwned, _get_sha1


def check(pasw, *args, **kwargs):
    """Checks a password.

    Parameters
    ----------
    pasw : str
        A password to check.
    """
    hashed = _get_sha1(pasw)
    digest = _get_hexdigest(hashed)
    prefix = digest[:5]
    suffix = digest[5:]
    pwned = _get_pwned(prefix)
    return _check_pwned(suffix, pwned)



"""
setup.py

https://github.com/navdeep-G/setup.py
"""
import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = "powned"
DESCRIPTION = (
    "A python package allowing you to check if your password has been pwned."
)
URL = "https://github.com/alexmacniven/powned"
EMAIL = "macniven.ap@gmail.com"
AUTHOR = "Alex Macniven"
REQUIRES_PYTHON = ">=3.7.0"
VERSION = "0.1.0"

# What packages are required for this module to be executed?
REQUIRED = [
    "requests"
]

# What packages are optional?
EXTRAS = {
    "dev": [
        "pytest",
        "rope",
        "flake8",
        "coverage"
    ]
}

# The rest you shouldn"t have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if "README.md" is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package"s __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, "__version__.py")) as f:
        exec(f.read(), about)
else:
    about["__version__"] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system(
            "{0} setup.py sdist bdist_wheel --universal".format(
                sys.executable
            )
        )

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(about["__version__"]))
        os.system("git push --tags")

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*"]
    ),
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license="MIT",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy"
    ],
    # $ setup.py publish support.
    cmdclass={
        "upload": UploadCommand,
    },
)
